package org.jpos.util;

import java.util.BitSet;

import junit.framework.TestCase;
import net.moneyswitch.common.easymock.EasyMockHelper;

import org.jpos.iso.*;

/**
 * $Revision: 23417 $
 * $Date: 2007-03-14 00:17:28 +0000 (Wed, 14 Mar 2007) $
 * $Author: nsmith $
 */
public class ISOComponentDeepCopierIntegrationTest extends TestCase {

    private ISOBinaryField testBinaryField;
    private ISOBinaryField anotherTestBinaryField;
    private ISOBitMap testBitMap;
    private ISOBitMap anotherTestBitMap;
    private ISOField testField;
    private ISOField anotherTestField;
    private ISOMsg testMsg;
    private ISOMsg anotherTestMsg;
    private ISOMsg testSubMsg;
    private ISOMsg anotherTestSubMsg;


    protected void setUp() throws Exception {
        EasyMockHelper.setupMocks(this);
        
        setupTestFields();
        setupOtherTestFields();
    }
    
    private void setupTestFields() throws Exception {
        testBinaryField = new ISOBinaryField(6, new byte[] {0x12, 0x78, (byte) 0x9C});

        BitSet bitSet = new BitSet(48);
        bitSet.set(11);
        bitSet.set(22);
        bitSet.set(33);
        testBitMap = new ISOBitMap(8, bitSet);

        testField = new ISOField(10, "testField");
        
        testMsg = new ISOMsg("0200");
        testMsg.set(testBitMap);
        testMsg.set(testBinaryField);
        testSubMsg = new ISOMsg(12);
        testSubMsg.set(testField);
        testMsg.set(testSubMsg);
    }
    
    private void setupOtherTestFields() throws Exception {
        anotherTestBinaryField = new ISOBinaryField(6, new byte[] {0x12, 0x78, (byte) 0x9C});

        BitSet anotherBitSet = new BitSet(48);
        anotherBitSet.set(11);
        anotherBitSet.set(22);
        anotherBitSet.set(33);
        anotherTestBitMap = new ISOBitMap(8, anotherBitSet);
        
        anotherTestField = new ISOField(10, "testField");
        
        anotherTestMsg = new ISOMsg("0200");
        anotherTestMsg.set(anotherTestBitMap);
        anotherTestMsg.set(anotherTestBinaryField);
        anotherTestSubMsg = new ISOMsg(12);
        anotherTestSubMsg.set(anotherTestField);
        anotherTestMsg.set(anotherTestSubMsg);
    }

    public void testDeepCopyOfISOMsg() throws Exception {
        EasyMockHelper.replayMocks(this);
        ISOMsgEqualityTester.assertEquals(anotherTestMsg, (ISOMsg) ISOComponentDeepCopier.getInstance().doDeepCopy(testMsg)); 
        EasyMockHelper.verifyMocks(this);
    }
}

