package org.jpos.util;

import java.util.BitSet;

import junit.framework.TestCase;
import net.moneyswitch.common.easymock.EasyMockHelper;

import org.easymock.ArgumentsMatcher;
import org.easymock.MockControl;
import org.jpos.iso.*;

/**
 * $Revision: 23416 $
 * $Date: 2007-03-14 00:14:09 +0000 (Wed, 14 Mar 2007) $
 * $Author: nsmith $
 */
public class ISOComponentDeepCopierTest extends TestCase {

    private ISOComponentDeepCopier isoComponentDeepCopier;
    
    protected ISOBinaryFieldDeepCopier isoBinaryFieldDeepCopier;
    protected MockControl isoBinaryFieldDeepCopierControl;

    protected ISOBitMapDeepCopier isoBitMapDeepCopier; 
    protected MockControl isoBitMapDeepCopierControl; 
    
    protected ISOFieldDeepCopier isoFieldDeepCopier; 
    protected MockControl isoFieldDeepCopierControl; 
    
    protected ISOMsgDeepCopier isoMsgDeepCopier; 
    protected MockControl isoMsgDeepCopierControl; 

    private ISOBinaryField testBinaryField;
    private ISOBinaryField anotherTestBinaryField;
    private ISOBitMap testBitMap;
    private ISOBitMap anotherTestBitMap;
    private ISOField testField;
    private ISOField anotherTestField;
    private ISOMsg testMsg;
    private ISOMsg anotherTestMsg;


    protected void setUp() throws Exception {
        EasyMockHelper.setupMocks(this);
        isoComponentDeepCopier = new ISOComponentDeepCopier();
        isoComponentDeepCopier.setIsoBinaryFieldDeepCopier(isoBinaryFieldDeepCopier);
        isoComponentDeepCopier.setIsoBitMapDeepCopier(isoBitMapDeepCopier);
        isoComponentDeepCopier.setIsoFieldDeepCopier(isoFieldDeepCopier);
        isoComponentDeepCopier.setIsoMsgDeepCopier(isoMsgDeepCopier);

        testBinaryField = new ISOBinaryField(6, new byte[] {0x12, 0x78, (byte) 0x9C});
        anotherTestBinaryField = new ISOBinaryField(7, new byte[] {0x13, 0x79, (byte) 0x9DC});

        BitSet bitSet = new BitSet(48);
        bitSet.set(11);
        bitSet.set(22);
        bitSet.set(33);
        testBitMap = new ISOBitMap(8, bitSet);
        BitSet anotherBitSet = new BitSet(46);
        anotherBitSet.set(12);
        anotherBitSet.set(23);
        anotherBitSet.set(34);
        anotherTestBitMap = new ISOBitMap(9, anotherBitSet);
        
        testField = new ISOField(10, "testField");
        anotherTestField = new ISOField(11, "anotherTestField");
        
        
        testMsg = new ISOMsg(12);
        testMsg.set(testBitMap);
        testMsg.set(testBinaryField);
        ISOMsg testSubMsg = new ISOMsg(4);
        testSubMsg.set(testField);
        testMsg.set(testSubMsg);
        
        anotherTestMsg = new ISOMsg(13);
        anotherTestMsg.set(anotherTestBitMap);
        anotherTestMsg.set(anotherTestBinaryField);
        ISOMsg anotherTestSubMsg = new ISOMsg(4);
        anotherTestSubMsg.set(anotherTestField);
        anotherTestMsg.set(anotherTestSubMsg);
    }
    
    public void testDoDeepCopyWithOnlyISOBinaryField() throws Exception {
        isoBinaryFieldDeepCopierControl.expectAndReturn(isoBinaryFieldDeepCopier.doDeepCopy(testBinaryField), anotherTestBinaryField);
        
        EasyMockHelper.replayMocks(this);
        ISOBinaryFieldEqualityTester.assertEquals(anotherTestBinaryField, (ISOBinaryField) isoComponentDeepCopier.doDeepCopy(testBinaryField));
        EasyMockHelper.verifyMocks(this);
    }
    
    public void testDoDeepCopyWithOnlyISOBitMap() throws Exception {
        isoBitMapDeepCopierControl.expectAndReturn(isoBitMapDeepCopier.doDeepCopy(testBitMap), anotherTestBitMap);
        
        EasyMockHelper.replayMocks(this);
        ISOComponentEqualityTester.assertEquals(anotherTestBitMap, (ISOBitMap) isoComponentDeepCopier.doDeepCopy(testBitMap));
        EasyMockHelper.verifyMocks(this);
    }
    
    public void testDoDeepCopyWithOnlyISOField() throws Exception {
        isoFieldDeepCopierControl.expectAndReturn(isoFieldDeepCopier.doDeepCopy(testField), anotherTestField);
        
        EasyMockHelper.replayMocks(this);
        ISOComponentEqualityTester.assertEquals(anotherTestField, isoComponentDeepCopier.doDeepCopy(testField));
        EasyMockHelper.verifyMocks(this);
    }
    
    public void testDoDeepCopyWithISOMsg() throws Exception {
        isoMsgDeepCopierControl.expectAndReturn(isoMsgDeepCopier.doDeepCopy(testMsg), anotherTestMsg);
        
        EasyMockHelper.replayMocks(this);
        ISOMsgEqualityTester.assertEquals(anotherTestMsg, (ISOMsg) isoComponentDeepCopier.doDeepCopy(testMsg));
        EasyMockHelper.verifyMocks(this);
    }

    public static final ISOComponentDeepCopierDoDeepCopyMatcher ISO_COMPONENT_DEEP_COPIER_DO_DEEP_COPY_MATCHER = new ISOComponentDeepCopierDoDeepCopyMatcher();
    
    private static final class ISOComponentDeepCopierDoDeepCopyMatcher implements ArgumentsMatcher {

        public boolean matches(Object[] arg0, Object[] arg1) {
            return ISOComponentStringifier.stringify((ISOComponent) arg0[0]).equals(ISOComponentStringifier.stringify((ISOComponent) arg0[0]));
        }

        public String toString(Object[] arg0) {
            return ISOComponentStringifier.stringify((ISOComponent) arg0[0]);
        }
        
    }
}

