package org.jpos.util;

import java.util.BitSet;

import junit.framework.TestCase;
import net.moneyswitch.common.easymock.EasyMockHelper;

import org.easymock.MockControl;
import org.jpos.iso.*;

/**
 * $Revision: 23416 $
 * $Date: 2007-03-14 00:14:09 +0000 (Wed, 14 Mar 2007) $
 * $Author: nsmith $
 */
public class ISOMsgDeepCopierTest extends TestCase {

    private ISOComponentDeepCopier isoComponentDeepCopier;
    private MockControl isoComponentDeepCopierControl;

    private ISOMsgDeepCopier copier;
    
    private ISOBinaryField testBinaryField;
    private ISOBinaryField anotherTestBinaryField;
    private ISOBitMap testBitMap;
    private ISOBitMap anotherTestBitMap;
    private ISOField testField;
    private ISOField anotherTestField;
    private ISOMsg testMsg;
    private ISOMsg anotherTestMsg;
    private ISOMsg testSubMsg;
    private ISOMsg anotherTestSubMsg;


    protected void setUp() throws Exception {
        EasyMockHelper.setupMocks(this);
        
        copier = new ISOMsgDeepCopier(isoComponentDeepCopier);
        
        testBinaryField = new ISOBinaryField(6, new byte[] {0x12, 0x78, (byte) 0x9C});
        anotherTestBinaryField = new ISOBinaryField(7, new byte[] {0x13, 0x79, (byte) 0x9DC});

        BitSet bitSet = new BitSet(48);
        bitSet.set(11);
        bitSet.set(22);
        bitSet.set(33);
        testBitMap = new ISOBitMap(8, bitSet);
        BitSet anotherBitSet = new BitSet(46);
        anotherBitSet.set(12);
        anotherBitSet.set(23);
        anotherBitSet.set(34);
        anotherTestBitMap = new ISOBitMap(9, anotherBitSet);
        
        testField = new ISOField(10, "testField");
        anotherTestField = new ISOField(11, "anotherTestField");
        
        testMsg = new ISOMsg();
        testMsg.set(testBitMap);
        testMsg.set(testBinaryField);
        testSubMsg = new ISOMsg(12);
        testSubMsg.set(testField);
        testMsg.set(testSubMsg);
        
        anotherTestMsg = new ISOMsg();
        anotherTestMsg.set(anotherTestBitMap);
        anotherTestMsg.set(anotherTestBinaryField);
        anotherTestSubMsg = new ISOMsg(13);
        anotherTestSubMsg.set(anotherTestField);
        anotherTestMsg.set(anotherTestSubMsg);
    }
    
    public void testDeepCopyOfISOMsg() throws Exception {
        isoComponentDeepCopier.doDeepCopy(testBitMap);
        isoComponentDeepCopierControl.setMatcher(ISOComponentDeepCopierTest.ISO_COMPONENT_DEEP_COPIER_DO_DEEP_COPY_MATCHER);
        isoComponentDeepCopierControl.setReturnValue(anotherTestBitMap);
        isoComponentDeepCopier.doDeepCopy(testBinaryField);
        isoComponentDeepCopierControl.setReturnValue(anotherTestBinaryField);
        isoComponentDeepCopier.doDeepCopy(testSubMsg);
        isoComponentDeepCopierControl.setReturnValue(anotherTestSubMsg);
        
        EasyMockHelper.replayMocks(this);
        ISOMsgEqualityTester.assertEquals(anotherTestMsg, copier.doDeepCopy(testMsg)); 
        EasyMockHelper.verifyMocks(this);
    }
}

